import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.Observable;
import java.util.Observer;
import com.jinsight.jetchart.*;


/**
 * This   demo  generates  a  gif  file  and  a  client-side  image   map  from  a  pie  chart.
 * After  clicking  the  'save gif'  button,  a  gif  image is created in the current directory
 * together with an html file. Load the html file from a browser to see the mapped image.
 * Check the method createSerieMap() below for further details.
 * Gif generation and image mapping is useful if  charts are generated  from  servlet requests.
 * The  gif  file  can  be  created  in the server-side by a servlet, which generates   an html
 * code containing the image map and a reference to the mapped gif file.
 * For this application  to work properly, the file 'sEncoders.jar', found in the JetChart/lib/
 * Swing directory, must be added to the classpath.
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */

public class Main extends JFrame implements ActionListener,Observer,Runnable {

    PieGraph graph;
    PieSerie ps;

    // This class is found in the Encoders.jar file.
    ChartEncoder chartEncoder;

    /**
     * A File object is created here. The  gif encoding process generates
     * a file in the application directory with the name piechart.gif.
     */
    File imageFile=new File("piechart.gif");

    boolean firstUpdate=true;

    JLabel gifLabel,gifPercent;
    int maximum;

    public Main() {
	setTitle("The JetChart Library - Generating a gif file and a image map");
	graph=new PieGraph();
	graph.setTitle(new String[]{"The JetChart Library",
					    "Generating a gif file and a image map"});

	// Creates chart encoder. The chart encoder wraps a GIF, a PNG and a JPEG encoders.
	// To start encoding chart, some methods must be invoked on this object. See the 
	// run() method.
	chartEncoder=new ChartEncoder(graph);

	/**
	 * An observer is informed about  gif generation process. 
	 * While pixels are sent to file, the observer is notified.
	 * See the end of the update method.
	 */
	chartEncoder.addImageEncodingObserver(this);

	// Sets graphic context properties
	setChartProperties();

	// Creates a pie series and sets its properties.
	createPieSerie();
	 
	// Adds pie serie to the graphic context
	graph.addSerie(ps);
	
        Container ct=getContentPane();

	ct.add("Center",graph);

	// Creates top panel with a gif generation button
	createTopPanel();

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent evt) {
		System.exit(0);
	    }
	});
	

	setSize(500,400);

	// Centers frame on the window
	Dimension d=getToolkit().getScreenSize();
	setLocation((d.width-getSize().width)/2,(d.height-getSize().height)/2);

	setVisible(true);
    }
    
    private void setChartProperties() {

	// Enables 3D
	graph.set3DEnabled(true);
	
	// Enables dragging
	graph.setDraggingEnabled(true);
	
	// Moves legend to the right
	graph.getLegend().setPosition(Legend.RIGHT);
    }

    private void createPieSerie() {

	// Values of each slice
	double[] sliceValues={100,50,90,40,200,60};
	// Titles of each slice
	String[] sliceTitles={"Javasoft","Jinsight","Javalobby","Javacat","Javashareware","Jars"};

	// Colors of each slice
	Color[] sliceColors={Color.yellow,Color.blue,Color.red,Color.green,Color.pink,Color.lightGray};

	// Creates pie series
 	ps=new PieSerie();

	for (int counter=0;counter<sliceValues.length;counter++) {
	    Slice slice=new Slice(sliceValues[counter],sliceTitles[counter],sliceColors[counter]);
	    ps.addSlice(slice);
	}

    }

    private void createTopPanel() {
	JPanel topPanel=new JPanel();
	topPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

	JButton gif=new JButton("Save gif");
	gif.addActionListener(this);
	topPanel.add(gif);

	gifLabel=new JLabel("Percentual of pixels sent to file:");
	gifPercent=new JLabel("    ");
	gifLabel.setVisible(false);
	gifPercent.setVisible(false);

	topPanel.add(gifLabel);
	topPanel.add(gifPercent);
	
        Container ct=getContentPane();
	ct.add("North",topPanel);
    }

    public void createSerieMap() {


	/**
	 * To generate a .map file, disable the lines marked with an asterisk
	 * and change the name of the file to be created  to another one with
	 * a .map extension. Change the map type to AbstractSerie.SERVER_SIDE_NCSA or
	 * AbstractSerie.SERVER_SIDE_CERN. 
	 
	 * In this example an html file is generated with a pie chart image mapped.
	 * In a production environment, the ideal solution is a servlet with JetChart
	 * creating gif images from client requests. The image map is loaded by a
	 * client browser, containing an image tag referring a servlet that generates
	 * the mapped image and send it back to the client browser on a servlet output
	 * stream.
	 */
	String serieMap="";
	// Creates an html text to insert serie map. 
	serieMap+="<html><body bgcolor=white><br><br>\n\n"; // (*)

	serieMap+="<map name='map'>\n"; //(*)
	
	// Gets pie series image map.
	serieMap+=ps.getSerieMap(new String[]{"http://www.javasoft.com","http://www.jinsight.com",
					      "http://www.javalobby.com","http://www.javacat.com",
					      "http://www.javashareware.com","http://www.jars.com"},
				 AbstractSerie.CLIENT_SIDE,null)+"\n";
    
	
	serieMap+="</map>\n"; // (*)
	
	// Links piechart.gif(the gif image generated in this example) to serie map.
	serieMap+="<br><br><image src=piechart.gif usemap='#map' border=0>\n";  // (*)

	// Ends html code.
	serieMap+="<br><br></body></html>"; // (*)

	// Writes html code to a file.
	try {
	    FileWriter fw=new FileWriter("piechart.html");
	    fw.write(serieMap);
	    fw.close();
	}
	catch (IOException e) {
	    System.out.println(e.getMessage());
	}

    }

   

    public void actionPerformed(ActionEvent evt) {

	createSerieMap();
	

	/**
	 * Saves chart image in file imageFile. The process will take effect in a thread, here,
	 * to demonstrate  how to generate visual information about the pixels already saved in a
	 * file.
	 */
	firstUpdate=true;
	gifLabel.setVisible(true);
	gifPercent.setVisible(true);
	validate();	

	/**
	 * Disables chart to prevent pie handling while generating gif. Other alternative is to set
	 * thread priority to 2, without disabling chart. But, in this case, gif generation will
	 * be slower.
	 */
	graph.setEnabled(false);

	Thread t=new Thread(this);
	t.start();
	
    }

    public void run() {
	/**
	 * Starts encoding process, calling the gifEncode method of the ChartEncoder class. 
	 * The second parameter is a scale percentage applied to the image being saved. Note
	 * that the gifEncode() method internally handles a gif encoder object, which sends
	 * to the update() method of the Observer object(in this case,this application) the
	 * total of pixels to  be saved and the number of pixels already saved. * See the
	 * update() method below. There is a small delay before the encoder send values to the
	 * observer. That's because it is calculating the total amount of pixels of the chart
	 * image. The 'chartEncoder' object was previously created in the constructor of this
	 * Main object.
	 */
	try {
	    chartEncoder.gifEncode(imageFile,100);
	    // A message dialog can be inserted here to inform user about ending of the encoding process.
	    // In a swing application, a JOptionPane dialog is adequate.
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	finally {
	    // Disables labels visibility and enables chart.
	    gifLabel.setVisible(false);
	    gifPercent.setVisible(false);
	    gifPercent.setText("    ");
	    graph.setEnabled(true);
	}

    }

    // Implements the Observer interface
    public void update(Observable obs, Object obj) {
	
	/**
	 * The first time the observer is notified, the image encoder  sends the amount of pixels
	 * that will be saved in file, as an Integer object.  Subsequent notifications will inform the amount of pixels
	 * already sent to file. This is useful when is necessary to show some visual information
	 * about the encoding process. The firstUpdate boolean variable below controls value
	 * detection. The Observable obs variable is a reference to the gif encoder object, and
	 * is not useful here.
	 */
	int value=((Integer)obj).intValue();

	if (firstUpdate) {
	    // This code block is executed only once.
	    firstUpdate=false;
	    // Registers total of pixels
	    maximum=value;
	}
	else {
	    // Shows percentage information
	    gifPercent.setText(Integer.toString(value*100/maximum)+" %");
	    
	}
		
    }

    public static void main(String[] args) {
	new Main();
    }

}
    
